package com.tuoheng.admin.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tuoheng.admin.entity.Inspection;
import com.tuoheng.admin.entity.InspectionFile;
import com.tuoheng.admin.entity.QuestionType;
import com.tuoheng.admin.entity.User;
import com.tuoheng.admin.enums.UserTypeEnum;
import com.tuoheng.admin.enums.code.inspectionfile.ListByDeptUserTypeEnum;
import com.tuoheng.admin.enums.code.questionType.QuestionTypeEnum;
import com.tuoheng.admin.mapper.*;
import com.tuoheng.admin.query.InspectionFileQuery;
import com.tuoheng.admin.request.inspectionfile.QueryInspectionFilePageListByInspectionIdRequest;
import com.tuoheng.admin.request.inspectionfile.QueryInspectionFilePageListRequest;
import com.tuoheng.admin.service.IInspectionFileService;
import com.tuoheng.admin.service.inspectionfile.confirm.InspectionFileConfirmService;
import com.tuoheng.admin.service.inspectionfile.ignore.InspectionFileIgnoreService;
import com.tuoheng.admin.service.inspectionfile.query.QueryInspectionFilePageListByInspectionIdService;
import com.tuoheng.admin.service.inspectionfile.query.QueryInspectionFilePageListService;
import com.tuoheng.admin.utils.ShiroUtils;
import com.tuoheng.admin.vo.InspectionFileVo;
import com.tuoheng.admin.vo.ListByDeptUserTypeVo;
import com.tuoheng.common.core.enums.ServiceExceptionEnum;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;


import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 任务问题管理 前端控制器
 *
 * @Author ChengWang
 * @Date 2022/11/29
 */
@Service
@Slf4j
public class InspectionFileServiceImpl implements IInspectionFileService {

    @Autowired
    private InspectionFileMapper inspectionFileMapper;

    @Autowired
    private InspectionMapper inspectionMapper;

    @Autowired
    private QuestionTypeMapper questionTypeMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private DeptMapper deptMapper;

    @Autowired
    private QueryInspectionFilePageListByInspectionIdService queryInspectionFilePageListByInspectionIdService;

    @Autowired
    private QueryInspectionFilePageListService queryInspectionFilePageListService;

    @Autowired
    private InspectionFileIgnoreService inspectionFileIgnoreService;

    @Autowired
    private InspectionFileConfirmService inspectionFileConfirmService;

    /**
     * 问题类型和任务名称
     *
     * @return
     */
    @Override
    public JsonResult getQuestionList() {
        //String tenantId = ShiroUtils.getTenantId();
        String tenantId = "0";
        //查询区域范范围问题
        List<InspectionFile> inspectionFileLists = new ArrayList<>();
        List<InspectionFile> inspectionFileList = inspectionFileMapper.selectList(Wrappers.<InspectionFile>lambdaQuery()
                .eq(InspectionFile::getMark, 1)
                .eq(InspectionFile::getTenantId, tenantId));
        if (null == inspectionFileList) {
            return JsonResult.error("该覆盖区域范围内没有问题");
        }
        for (InspectionFile inspectionFile : inspectionFileList) {
            //任务状态判断
            if(20 == inspectionFile.getStatus()){
                //任务状态判断，已生成工单，问题未解决
                Inspection inspection = inspectionMapper.selectById(inspectionFile.getInspectionId());
                if (ObjectUtil.isNull(inspection)) {
                    JsonResult.error("问题所属的任务不存在");
                }
                inspectionFileLists.add(inspectionFile);
            }
        }
        List<InspectionFileVo> list = inspectionFileLists.stream().map(x -> {
            InspectionFileVo vo = new InspectionFileVo();
            BeanUtils.copyProperties(x, vo);
            //任务名称
            if (StringUtils.isNotEmpty(x.getInspectionId())) {
                Inspection inspection = inspectionMapper.selectById(x.getInspectionId());
                if (ObjectUtil.isNull(inspection)) {
                    JsonResult.error();
                }
                vo.setName(inspection.getName());
                //问题类型
                if (StringUtils.isNotEmpty(x.getQuestionId())) {
                    QuestionType questionType = questionTypeMapper.selectById(x.getQuestionId());
                    if (null != questionType) {
                        vo.setType(questionType.getName());
                    }
                }
            }

            return vo;
        }).collect(Collectors.toList());
        return JsonResult.success(list);
    }

    @Override
    public JsonResult getListByDeptUserType(InspectionFileQuery query) {
        if(query.getUserId()==null){
            JsonResult.error(ListByDeptUserTypeEnum.QUERY_IS_FAILED.getCode(),ListByDeptUserTypeEnum.QUERY_IS_FAILED.getMsg());
        }
        //获取当前登录人信息
        //String username = SecurityUserUtils.username();
        String username = "admin";
        if(StringUtils.isEmpty(username)){
            JsonResult.error(ListByDeptUserTypeEnum.USER_NAME_IS_NULL.getCode(),ListByDeptUserTypeEnum.USER_NAME_IS_NULL.getMsg());
        }
        User user = userMapper.selectOne(new LambdaQueryWrapper<User>()
                .eq(User::getMark, 1).eq(User::getUsername, username));
        if(ObjectUtil.isNull(user)){
            JsonResult.error(ListByDeptUserTypeEnum.USER_IS_NULL.getCode(),ListByDeptUserTypeEnum.USER_IS_NULL.getMsg());
        }
        String tenantId = ShiroUtils.getTenantId();
        //用户角色判断 1超级管理员 2部门管理员 3普通用户
        if(null == user.getType()){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        List<ListByDeptUserTypeVo> list = new ArrayList<>();
        ListByDeptUserTypeVo vo = new ListByDeptUserTypeVo();
        ListByDeptUserTypeVo vo1 = new ListByDeptUserTypeVo();
        ListByDeptUserTypeVo vo2 = new ListByDeptUserTypeVo();
        //问题数量
        Long pitGrooveNum = 0l;
        Long ponDingNum = 0l;
        Long crackNum = 0l;
        //若角色为超级管理员，查看状态为已生成工单和和问题已处理
        if(UserTypeEnum.SUPER_ADMIN.getCode()==user.getType()){
            //直接查问题列表
            List<InspectionFile> inspectionFileList = inspectionFileMapper.selectList(Wrappers.<InspectionFile>lambdaQuery()
                    .eq(InspectionFile::getMark, 1)
                    .eq(InspectionFile::getTenantId, tenantId)
                    .in(InspectionFile::getStatus,20,25));
            //根据状态类型分类
            if(null == inspectionFileList){
                JsonResult.error(ListByDeptUserTypeEnum.INSPECTION_TYPE_LIST_IS_NULL.getCode(),ListByDeptUserTypeEnum.INSPECTION_TYPE_LIST_IS_NULL.getMsg());
            }
            for (InspectionFile inspectionFile : inspectionFileList) {
                if(StringUtils.isEmpty(inspectionFile.getQuestionId())){
                    JsonResult.error(ListByDeptUserTypeEnum.QUESTION_ID_IS_NULL.getCode(),ListByDeptUserTypeEnum.QUESTION_ID_IS_NULL.getMsg());
                }
                //查找问题类型
                QuestionType questionType = questionTypeMapper.selectOne(Wrappers.<QuestionType>lambdaQuery()
                        .eq(QuestionType::getMark, 1)
                        .eq(QuestionType::getId, inspectionFile.getQuestionId()));
                if(ObjectUtil.isNull(questionType)){
                    throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
                }
                Integer name = questionType.getName();
                //1坑槽
                if(name== QuestionTypeEnum.PING_GROOVE_NAME.getCode()) {
                    pitGrooveNum+=1;
                }else if(name==QuestionTypeEnum.PON_DING_NAME.getCode()){
                    //积水
                    ponDingNum+=1;
                }else if(name==QuestionTypeEnum.CRACK_NAME.getCode()){
                    //裂缝
                    crackNum+=1;
                }
            }
            vo.setType(QuestionTypeEnum.PING_GROOVE_NAME.getCode());
            vo1.setType(QuestionTypeEnum.PON_DING_NAME.getCode());
            vo2.setType(QuestionTypeEnum.CRACK_NAME.getCode());
            vo.setNum(pitGrooveNum);
            vo1.setNum(ponDingNum);
            vo2.setNum(crackNum);
            list.add(vo);
            list.add(vo1);
            list.add(vo2);
        }
        //若角色为部门管理员或普通用户
        if(UserTypeEnum.ADMIN.getCode()== user.getType() || UserTypeEnum.ORDINARY_USER.getCode()== user.getType()){
            //获取用户对应的部门
            String deptId = user.getDeptId();
            if(StringUtils.isEmpty(deptId)){
                JsonResult.error(ListByDeptUserTypeEnum.DEPT_ID_IS_NULL.getCode(),ListByDeptUserTypeEnum.DEPT_ID_IS_NULL.getMsg());
            }
            //根据部门id获取部门id列表
            List<String> deptIdList = deptMapper.selectAllChildListById(deptId);
            if(CollectionUtil.isEmpty(deptIdList)){
                return null;
            }
            //根据部门id列表查多条任务
            List<Inspection> inspectionList = inspectionMapper.selectListByDeptIdList(deptIdList);
            if(null == inspectionList){
                JsonResult.error(ListByDeptUserTypeEnum.INSPECTION_LIST_IS_NULL.getCode(),ListByDeptUserTypeEnum.INSPECTION_LIST_IS_NULL.getMsg());
            }
            List<String> inspectionIdList = inspectionList.stream().map(o -> o.getId()).collect(Collectors.toList());
            //根据任务id列表查找多条问题
            List<InspectionFile> inspectionFileList = inspectionFileMapper.selectListByInspectIdList(inspectionIdList);
            //根据状态类型分类
            if(null == inspectionFileList){
                JsonResult.error(ListByDeptUserTypeEnum.INSPECTION_TYPE_LIST_IS_NULL.getCode(),ListByDeptUserTypeEnum.INSPECTION_TYPE_LIST_IS_NULL.getMsg());
            }
            for (InspectionFile inspectionFile : inspectionFileList) {
                if(StringUtils.isEmpty(inspectionFile.getQuestionId())){
                    JsonResult.error(ListByDeptUserTypeEnum.QUESTION_ID_IS_NULL.getCode(),ListByDeptUserTypeEnum.QUESTION_ID_IS_NULL.getMsg());
                }
                //查找问题类型
                QuestionType questionType = questionTypeMapper.selectOne(Wrappers.<QuestionType>lambdaQuery()
                        .eq(QuestionType::getMark, 1)
                        .eq(QuestionType::getId, inspectionFile.getQuestionId()));
                if(ObjectUtil.isNull(questionType)){
                    throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
                }
                Integer name = questionType.getName();
                //1坑槽
                if(name==QuestionTypeEnum.PING_GROOVE_NAME.getCode()) {
                    pitGrooveNum+=1;
                }else if(name==QuestionTypeEnum.PON_DING_NAME.getCode()){
                    //积水
                    ponDingNum+=1;
                }else if(name==QuestionTypeEnum.CRACK_NAME.getCode()){
                    //裂缝
                    crackNum+=1;
                }
            }
            vo.setType(QuestionTypeEnum.PING_GROOVE_NAME.getCode());
            vo1.setType(QuestionTypeEnum.PON_DING_NAME.getCode());
            vo2.setType(QuestionTypeEnum.CRACK_NAME.getCode());
            vo.setNum(pitGrooveNum);
            vo1.setNum(ponDingNum);
            vo2.setNum(crackNum);
            list.add(vo);
            list.add(vo1);
            list.add(vo2);

        }

        return JsonResult.success(list);
    }

    /**
     * 根据任务ID查询任务问题分页列表
     *
     * @return
     */
    @Override
    public JsonResult getPageListByInspectionId(QueryInspectionFilePageListByInspectionIdRequest request){
        return queryInspectionFilePageListByInspectionIdService.getPageListByInspectionId(request);
    }

    /**
     *
     * 查询任务问题分页列表
     *
     * @param request
     * @return
     */
    @Override
    public JsonResult getPageList(QueryInspectionFilePageListRequest request) {
        return queryInspectionFilePageListService.getPageList(request);
    }

    /**
     *
     * 确认
     *
     * @param idList
     * @return
     */
    @Override
    public JsonResult confirm(List<String> idList) {
        return inspectionFileConfirmService.confirm(idList);
    }

    /**
     *
     * 忽略
     *
     * @param idList
     * @return
     */
    @Override
    public JsonResult ignore(List<String> idList) {
        return inspectionFileIgnoreService.ignore(idList);
    }
}
