package com.tuoheng.miniprogram.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tuoheng.common.core.config.common.CommonConfig;
import com.tuoheng.common.core.enums.ServiceExceptionEnum;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.CommonUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.SecurityUserUtils;
import com.tuoheng.common.core.utils.StringUtils;
import com.tuoheng.miniprogram.dao.*;
import com.tuoheng.miniprogram.entity.*;
import com.tuoheng.miniprogram.entity.dto.QuestionIdentificationDto;
import com.tuoheng.miniprogram.entity.query.InspectionFileQuery;
import com.tuoheng.miniprogram.service.IInspectionFileService;
import com.tuoheng.miniprogram.vo.InspectionFileInfoVo;
import com.tuoheng.miniprogram.vo.SeeQuestionVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Author ChengWang
 * @Date 2022/11/23
 */
@Service
@Slf4j
public class InspectionFileServiceImpl implements IInspectionFileService {

    @Autowired
    private InspectionMapper inspectionMapper;

    @Autowired
    private InspectionFileMapper inspectionFileMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RoadInformationMapper roadInformationMapper;

    @Autowired
    private QuestionTypeMapper questionTypeMapper;

    /**
     *待确认问题 status状态为5
     * @param query
     * @return
     */
    @Override
    public JsonResult unconfirmedQuestion(InspectionFileQuery query) {
        if(null==query.getLimit() && null == query.getPage()){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        if(StringUtils.isEmpty(query.getInspectionId())){
            return JsonResult.error("任务id为空");
        }
        //获取当前登录人
//        String username = SecurityUserUtils.username();
//        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery()
//                .eq(User::getUsername, username)
//                .eq(User::getMark,1));
        //查询当前对应的任务
        Inspection inspection = inspectionMapper.selectOne(Wrappers.<Inspection>lambdaQuery()
                .eq(Inspection::getId, query.getInspectionId())
                .eq(Inspection::getMark, 1));
        if(ObjectUtil.isNull(inspection)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //设置分页
        IPage<InspectionFile> page = new Page<>(query.getPage(),query.getLimit());
        //获取分页数据
        IPage<InspectionFile> result = inspectionFileMapper.selectPage(page, Wrappers.<InspectionFile>lambdaQuery()
                .eq(InspectionFile::getInspectionId, query.getInspectionId())
                .eq(InspectionFile::getMark, 1)
                .eq(InspectionFile::getStatus, 5));
        IPage<InspectionFileInfoVo> pageData = getInspectionFileInfoVoIPage(inspection, result);

        return JsonResult.success(pageData);
    }

    /**
     * 已确认问题 status状态为15
     * @param query
     * @return
     */
    @Override
    public JsonResult confirmedQuestion(InspectionFileQuery query) {
        if(null==query.getLimit() && null == query.getPage()){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        if(StringUtils.isEmpty(query.getInspectionId())){
            return JsonResult.error("任务id为空");
        }
        //获取当前登录人
//        String username = SecurityUserUtils.username();
//        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery()
//                .eq(User::getUsername, username)
//                .eq(User::getMark,1));
        //查询当前对应的任务
        Inspection inspection = inspectionMapper.selectOne(Wrappers.<Inspection>lambdaQuery()
                .eq(Inspection::getId, query.getInspectionId())
                .eq(Inspection::getMark, 1));
        if(ObjectUtil.isNull(inspection)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //设置分页
        IPage<InspectionFile> page = new Page<>(query.getPage(),query.getLimit());
        //获取分页数据
        IPage<InspectionFile> result = inspectionFileMapper.selectPage(page, Wrappers.<InspectionFile>lambdaQuery()
                .eq(InspectionFile::getInspectionId, query.getInspectionId())
                .eq(InspectionFile::getMark, 1)
                .eq(InspectionFile::getStatus, 15));
        IPage<InspectionFileInfoVo> pageData = getInspectionFileInfoVoIPage(inspection, result);

        return JsonResult.success(pageData);

    }

    /**
     * 已忽略问题 status状态为10
     * @param query
     * @return
     */
    @Override
    public JsonResult ignoredQuestion(InspectionFileQuery query) {
        if(null==query.getLimit() && null == query.getPage()){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        if(StringUtils.isEmpty(query.getInspectionId())){
            return JsonResult.error("任务id为空");
        }
        //获取当前登录人
//        String username = SecurityUserUtils.username();
//        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery()
//                .eq(User::getUsername, username)
//                .eq(User::getMark,1));
        //查询当前对应的任务
        Inspection inspection = inspectionMapper.selectOne(Wrappers.<Inspection>lambdaQuery()
                .eq(Inspection::getId, query.getInspectionId())
                .eq(Inspection::getMark, 1));
        if(ObjectUtil.isNull(inspection)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //设置分页
        IPage<InspectionFile> page = new Page<>(query.getPage(),query.getLimit());
        //获取分页数据
        IPage<InspectionFile> result = inspectionFileMapper.selectPage(page, Wrappers.<InspectionFile>lambdaQuery()
                .eq(InspectionFile::getInspectionId, query.getInspectionId())
                .eq(InspectionFile::getMark, 1)
                .eq(InspectionFile::getStatus, 10));
        IPage<InspectionFileInfoVo> pageData = getInspectionFileInfoVoIPage(inspection, result);

        return JsonResult.success(pageData);

    }

    /**
     * 问题标识
     * @param dto
     * @return
     */
    @Override
    public JsonResult questionIdentification(QuestionIdentificationDto dto) {
        if(StringUtils.isEmpty(dto.getId())) return JsonResult.error("问题id不存在");
        //查询任务此问题
        InspectionFile inspectionFile = inspectionFileMapper.selectOne(Wrappers.<InspectionFile>lambdaQuery()
                .eq(InspectionFile::getId, dto.getId())
                .eq(InspectionFile::getMark, 1)
                .eq(StringUtils.isNotEmpty(dto.getTenantId()), InspectionFile::getTenantId, dto.getTenantId()));
        //修改对应的问题类型
        inspectionFile.setQuestionId(dto.getQuestionId());
        int count = inspectionFileMapper.updateById(inspectionFile);
        if(count<=0){
            return JsonResult.error();
        }
        //设置图片添加域名
        if(StringUtils.isNotEmpty(inspectionFile.getFileThumbnail())){
            inspectionFile.setFileThumbnail(CommonConfig.imageURL+inspectionFile.getFileThumbnail());
        }
        if(StringUtils.isNotEmpty(inspectionFile.getFileOriginal())){
            inspectionFile.setFileOriginal(CommonConfig.imageURL+inspectionFile.getFileOriginal());
        }
        if(StringUtils.isNotEmpty(inspectionFile.getFileImage())){
            inspectionFile.setFileImage(CommonConfig.imageURL+inspectionFile.getFileImage());
        }


        return JsonResult.success(inspectionFile);
    }

    /**
     * 确认问题
     * @param entity
     * @return
     */
    @Override
    public JsonResult confirm(InspectionFile entity) {
        if(StringUtils.isEmpty(entity.getId())) {
            return JsonResult.error("问题id为空");
        }
        //查询当前问题
        LambdaUpdateWrapper<InspectionFile> qw = new LambdaUpdateWrapper<>();
        qw.eq(InspectionFile::getId,entity.getId());
        qw.eq(InspectionFile::getMark,1);
        qw.set(InspectionFile::getStatus,15);
        int count = inspectionFileMapper.update(entity, qw);
        if(count<=0){
            return JsonResult.error();
        }
        return JsonResult.success("确认问题成功");
    }

    /**
     * 忽略问题
     * @param entity
     * @return
     */
    @Override
    public JsonResult ignore(InspectionFile entity) {
        if(null == entity.getId()) {
            return JsonResult.error("问题id为空");
        }
        //查询当前问题
        LambdaUpdateWrapper<InspectionFile> qw = new LambdaUpdateWrapper<>();
        qw.eq(InspectionFile::getId,entity.getId());
        qw.eq(InspectionFile::getMark,1);
        qw.set(InspectionFile::getStatus,10);
        int count = inspectionFileMapper.update(entity, qw);
        if(count<=0){
            return JsonResult.error();
        }
        return JsonResult.success("忽略问题成功");
    }

    /**
     * 查看
     * @param id
     * @return
     */
    @Override
    public JsonResult seeQuestion(String id) {
        //校验
        if(StringUtils.isEmpty(id)){
           throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        SeeQuestionVo vo = inspectionFileMapper.selectByInspectionId(id);
        if(ObjectUtil.isNull(vo)){
            return JsonResult.error("未查询出对应的问题及处理结果");
        }
        //对属性处理
        if(StringUtils.isNotEmpty(vo.getFileOriginal())){
            vo.setFileOriginal(CommonConfig.imageURL+vo.getFileOriginal());
        }
        if(StringUtils.isNotEmpty(vo.getFileImage())){
            vo.setFileImage(CommonConfig.imageURL+vo.getFileImage());
        }
        if(StringUtils.isNotEmpty(vo.getFileThumbnail())){
            vo.setFileThumbnail(CommonConfig.imageURL+vo.getFileThumbnail());
        }
        //对多张处理结果图进行处理用,隔开
        if(StringUtils.isNotEmpty(vo.getHandlerImage())){
            String getHandlerImage = getMultipleUrl(vo.getHandlerImage());
            vo.setHandlerImage(getHandlerImage);
        }


        return JsonResult.success(vo);
    }

    /**
     * 多个url进行转换,用,号隔开
     *
     * @param url
     * @return
     */
    private String getMultipleUrl(String url) {

        List<String> collect = Arrays.asList(url.split(",")).stream().map(
                s -> CommonUtils.getImageURL(s)).collect(Collectors.toList());
        //返回
        return org.apache.commons.lang3.StringUtils.join(collect, ",");
    }

    private IPage<InspectionFileInfoVo> getInspectionFileInfoVoIPage(Inspection inspection, IPage<InspectionFile> result) {
        //数据转换
        return result.convert(t -> {
            InspectionFileInfoVo vo = Convert.convert(InspectionFileInfoVo.class, t);
            //查询其他属性
            //公路代号
            RoadInformation roadInformation = roadInformationMapper.selectOne(Wrappers.<RoadInformation>lambdaQuery()
                    .eq(RoadInformation::getId, inspection.getRoadId()));
            if(ObjectUtil.isNotNull(roadInformation)){
                vo.setCode(roadInformation.getCode());
            }
            //任务名称
            if (StringUtils.isNotEmpty(inspection.getName())) {
                vo.setName(inspection.getName());
            }
            //任务时间
            if (null != inspection.getExecutionStartTime()){
                vo.setExecutionStartTime(inspection.getExecutionStartTime());
            }
            //缩略图
            if (StringUtils.isNotEmpty(t.getFileThumbnail())) {
                vo.setFileThumbnail(CommonConfig.imageURL + t.getFileThumbnail());
            }
            //问题类型
            if(StringUtils.isNotEmpty(t.getQuestionId())){
                QuestionType questionType = questionTypeMapper.selectOne(Wrappers.<QuestionType>lambdaQuery()
                        .eq(QuestionType::getId, t.getQuestionId())
                        .eq(QuestionType::getMark, 1));
                if(null == questionType){
                    throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
                }
                if(null != questionType.getName()){
                    vo.setQuestionName(questionType.getName());
                }
            }

            return vo;
        });
    }
}
