package com.tuoheng.miniprogram.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tuoheng.common.core.config.common.CommonConfig;
import com.tuoheng.common.core.enums.ServiceExceptionEnum;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.*;
import com.tuoheng.miniprogram.dao.*;
import com.tuoheng.miniprogram.entity.*;
import com.tuoheng.miniprogram.entity.query.InspectionQuery;
import com.tuoheng.miniprogram.enums.UserTypeEnum;
import com.tuoheng.miniprogram.service.IInspectionService;
import com.tuoheng.miniprogram.utils.ShiroUtils;
import com.tuoheng.miniprogram.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;


import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author ChengWang
 * @Date 2022/11/21
 */
@Service
@Slf4j
public class InspectionServiceImpl implements IInspectionService {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private DeptMapper deptMapper;

    @Autowired
    private InspectionMapper inspectionMapper;

    @Autowired
    private InspectionFileMapper inspectionFileMapper;

    @Autowired
    private TenantMapper tenantMapper;

    /**
     * 任务列表(分页)
     * @param query
     * @return
     */
    @Override
    public JsonResult index(InspectionQuery query) {
        if(null==query.getLimit() && null == query.getPage()){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //获取登录人信息,登录人用户名
//        String username = SecurityUserUtils.username();
        //获取租户id
        String tenantId = ShiroUtils.getTenantId();
        String username = "admin";
        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery()
                .eq(StringUtils.isNotEmpty(username), User::getUsername, username)
                .eq(User::getStatus, 1).eq(User::getMark, 1));
        //查询部门及下级部门列表
        List<Dept> list = deptMapper.selectList(Wrappers.<Dept>lambdaQuery()
                .eq(Dept::getMark, 1));
        //初始部门id
        String deptIdInt = query.getDeptId();
        //query.setDeptId("1");
        //query.setTenantId("0");
        //获取当前部门对应的巡检任务
//        List<Inspection> inspectionList = inspectionMapper.selectList(Wrappers.<Inspection>lambdaQuery()
//                .eq(StringUtils.isNotEmpty(query.getDeptId()), Inspection::getDeptId, query.getDeptId())
//                .eq(Inspection::getMark, 1)
//                .eq(Inspection::getTenantId,0));
        //获取分页数据
        IPage<Inspection> page = new Page<>(query.getPage(),query.getLimit());
        IPage<InspectionInfoVo> pageDataVo = new Page<>(query.getPage(),query.getLimit());
        Date startTime = null;
        Date endTime = null;
        if(StringUtils.isNotEmpty(query.getStartTime()) && StringUtils.isNotEmpty(query.getEndTime())){
            startTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,query.getStartTime()+" 00:00:00");
            endTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,query.getEndTime()+" 23:59:59");
        }
        query.setStartTimeDate(startTime);
        query.setEndTimeDate(endTime);
        Integer type = user.getType();
        if(null == type){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        List<InspectionInfoVo> inspectionInfoVoList = new ArrayList<>();
        //用户角色判断 1超级管理员 2部门管理员 3普通用户
        if(UserTypeEnum.SUPER_ADMIN.getCode()==type){
            IPage<InspectionInfoVo> IPageData = inspectionMapper.queryPage(page, query);
            IPageData.getRecords().stream().forEach(x->{
                int problemsFoundNum = 0;
                int problemsVerifiedNum = 0;
                List<InspectionFile> inspectionFileList = inspectionFileMapper.selectList(Wrappers.<InspectionFile>lambdaQuery()
                        .eq(StringUtils.isNotEmpty(x.getId()), InspectionFile::getInspectionId, x.getId())
                        .eq(InspectionFile::getTenantId, 0)
                        .eq(InspectionFile::getMark,1));
                if(StringUtils.isEmpty(inspectionFileList)){
                    throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
                }
                problemsVerifiedNum = inspectionFileList.size();
                for (InspectionFile inspectionFile : inspectionFileList) {
                    if (15 == inspectionFile.getStatus()) {
                        problemsFoundNum += 1;
                    }
                }
                x.setFlag(1);
                x.setProblemsFoundNum(problemsFoundNum);
                x.setProblemsVerifiedNum(problemsVerifiedNum);
            });
            List<InspectionInfoVo> records = IPageData.getRecords();
            for (InspectionInfoVo record : records) {
                inspectionInfoVoList.add(record);
            }

        }
        //登录用户角色为部门管理员或普通用户
        if(UserTypeEnum.ADMIN.getCode()==type || UserTypeEnum.ORDINARY_USER.getCode()==type){
            // 根据部门id获取此部门id和所有下级部门id封装list集合
            List<String> deptLists = deptMapper.selectAllChildListById(query.getDeptId());
            if (StringUtils.isEmpty(deptLists)){
                throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
            }
            for (int i = 0; i < deptLists.size(); i++) {
                String deptId = deptLists.get(i);
                query.setDeptId(deptId);
                IPage<InspectionInfoVo> pageData = inspectionMapper.queryPage(page,query);
                pageData.getRecords().stream().forEach(x->{
                    int problemsFoundNum = 0;
                    int problemsVerifiedNum = 0;
                    List<InspectionFile> inspectionFileList = inspectionFileMapper.selectList(Wrappers.<InspectionFile>lambdaQuery()
                            .eq(StringUtils.isNotEmpty(x.getId()), InspectionFile::getInspectionId, x.getId())
                            .eq(InspectionFile::getTenantId, 0)
                            .eq(InspectionFile::getMark,1));
                    if(StringUtils.isEmpty(inspectionFileList)){
                        throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
                    }
                    problemsVerifiedNum = inspectionFileList.size();
                    for (InspectionFile inspectionFile : inspectionFileList) {
                        if (15 == inspectionFile.getStatus()) {
                            problemsFoundNum += 1;
                        }
                    }
                    //获取任务deptId1:1为部门任务 0为子部门任务
                    if(deptIdInt.equals(deptId)){
                        x.setFlag(1);
                    }else {
                        x.setFlag(0);
                    }
                    x.setProblemsFoundNum(problemsFoundNum);
                    x.setProblemsVerifiedNum(problemsVerifiedNum);
                });
                List<InspectionInfoVo> records = pageData.getRecords();
                for (InspectionInfoVo record : records) {
                    inspectionInfoVoList.add(record);
                }
            }
        }

        pageDataVo.setRecords(inspectionInfoVoList);


        return JsonResult.success(pageDataVo);
    }


    @Override
    public JsonResult playback(String id) {
        //参数校验
        if(StringUtils.isEmpty(id)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        //根据任务id获取任务
        Inspection inspection = inspectionMapper.selectOne(Wrappers.<Inspection>lambdaQuery()
                .eq(Inspection::getTenantId, 0)
                .eq(Inspection::getMark, 1)
                .eq(Inspection::getId, id));
        PlayBackInfoVo vo = new PlayBackInfoVo();
        BeanUtils.copyProperties(inspection,vo);
        //对视频地址进行处理
        if(StringUtils.isNotEmpty(inspection.getAiVideoUrl())){
            vo.setAiVideoUrl(CommonConfig.videoURL+inspection.getVideoUrl());
        }

        return JsonResult.success(vo);
    }

    @Override
    public JsonResult deleteById(String id) {
        if (id == null || StringUtils.isEmpty(id)) {
            return JsonResult.error("记录ID不能为空");
        }
        // 设置Mark=0
        Inspection inspection = inspectionMapper.selectOne(Wrappers.<Inspection>lambdaQuery()
                .eq(Inspection::getId, id));
        inspection.setMark(0);
        //判断当前任务下的问题是否存在，存在则删除
        List<InspectionFile> inspectionFiles = inspectionFileMapper.selectList(new LambdaQueryWrapper<InspectionFile>()
                .eq(InspectionFile::getInspectionId, id));
        if(StringUtils.isEmpty(inspectionFiles)){
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }
        Integer num = 0;
        for (InspectionFile inspectionFile : inspectionFiles) {
            inspectionFile.setMark(0);
            int i = inspectionFileMapper.updateById(inspectionFile);
            num += i;
        }
        int count = inspectionMapper.updateById(inspection);
        if(count<=0 && num !=inspectionFiles.size()){
            return JsonResult.error();
        }

        return JsonResult.success("删除成功");

    }


    /**
     * 递归获取部门的子集
     *
     * @author zhu_zishuang
     * @date 3/13/21
     */
    private List<Dept> getChildren(Dept rootDept, List<Dept> list) {
        return list.stream().filter(dept ->
                dept.getPid().equals(rootDept.getId())
        ).peek(dept -> {
            // 设置子集
            dept.setItemList(getChildren(dept, list));
        }).sorted(Comparator.comparingInt(dept -> (dept.getSort() == null ? 0 : dept.getSort()))).collect(Collectors.toList());
    }
}
